<?php

namespace Drupal\webform_booking\Plugin\WebformElement;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\webform\Plugin\WebformElementBase;
use Drupal\webform\WebformSubmissionInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a 'webform_booking' element.
 *
 * @WebformElement(
 *   id = "webform_booking",
 *   label = @Translation("Booking"),
 *   description = @Translation("Provides a webform element for scheduling appointments."),
 *   category = @Translation("Booking"),
 * )
 *
 * @FormElement("webform_webform_booking")
 *
 * @see \Drupal\webform\Plugin\WebformElementBase
 * @see \Drupal\webform\Plugin\WebformElementInterface
 * @see \Drupal\webform\Annotation\WebformElement
 */
class WebformBooking extends WebformElementBase {
  use StringTranslationTrait;

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    $instance = parent::create($container, $configuration, $plugin_id, $plugin_definition);
    $instance->configFactory = $container->get('config.factory');
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  protected function defineDefaultProperties() {
    $properties = parent::defineDefaultProperties() + [
      'title' => '',
      'start_date' => '',
      'end_date' => '',
      'exclusion_dates' => '',
      'days_advance' => '0',
      'excluded_weekdays' => [],
      'time_interval' => '9:00|16:30',
      'slot_duration' => 60,
      'seats_slot' => '1',
      'max_seats_per_booking' => 1,
      'no_slots' => 'No slots available.',
      'paypal_enabled' => FALSE,
      'default_price' => 0,
      'excluded_time_periods' => '',
      'date_label' => '',
      'slot_label' => '',
      'seats_label' => '',
    ];
    unset($properties['markup']);
    return $properties;
  }

  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state) {
    $form = parent::form($form, $form_state);

    $form['#attached']['library'][] = 'webform_booking/webform_booking_element';

    unset($form['markup'], $form['default']);
    $form['form']['#access'] = FALSE;

    $form['appointment_settings'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Webform Booking Settings'),
    ];

    $form['appointment_settings']['title'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Title'),
      '#default_value' => $this->configuration['title'] ?? '',
    ];

    $form['appointment_settings']['date_range'] = [
      '#type' => 'container',
    ];

    $form['appointment_settings']['date_range']['start_date'] = [
      '#type' => 'date',
      '#title' => $this->t('Start Date'),
      '#default_value' => $this->configuration['start_date'] ?? '',
      '#prefix' => '<div class="booking-inline-field">',
    ];

    $form['appointment_settings']['date_range']['end_date'] = [
      '#type' => 'date',
      '#title' => $this->t('End Date'),
      '#default_value' => $this->configuration['end_date'] ?? '',
      '#suffix' => '</div>',
    ];

    $form['appointment_settings']['time_interval'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Time Intervals'),
      '#description' => $this->t('Enter time intervals in 24h format. One interval per line. Ex. 08:00|12:00. You can also add exceptions for specific days or weekdays (weekdays in English). Ex: 10:00|12:00(Friday) or 10:00|12:00(2024-10-04)'),
      '#default_value' => $this->configuration['time_interval'] ?? '9:00|16:30',
      '#required' => TRUE,
      '#element_validate' => [[get_class($this), 'validateTimeIntervals']],
      '#rows' => 2,
    ];

    $form['appointment_settings']['slot_duration'] = [
      '#type' => 'number',
      '#title' => $this->t('Slot Duration (min)'),
      '#description' => $this->t('In Minutes'),
      '#default_value' => $this->configuration['slot_duration'] ?? 60,
      '#min' => 1,
      '#required' => TRUE,
    ];

    $form['appointment_settings']['seats_slot'] = [
      '#type' => 'number',
      '#title' => $this->t('Seats Per Slot'),
      '#description' => $this->t('Number of bookings for the same slot'),
      '#default_value' => $this->configuration['seats_slot'] ?? 1,
      '#min' => 1,
      '#required' => TRUE,
      '#prefix' => '<div class="booking-inline-field">',
    ];

    $form['appointment_settings']['max_seats_per_booking'] = [
      '#type' => 'number',
      '#title' => $this->t('Max Seats Per Booking'),
      '#description' => $this->t('Maximum number of seats that can be booked in a single reservation'),
      '#default_value' => $this->configuration['max_seats_per_booking'] ?? 1,
      '#min' => 1,
      '#required' => TRUE,
      '#suffix' => '</div>',
    ];

    $form['advanced_options'] = [
      '#type' => 'details',
      '#title' => $this->t('Advanced options'),
      '#open' => FALSE,
      '#weight' => 43,
    ];

    $form['advanced_options']['days_advance'] = [
      '#type' => 'number',
      '#title' => $this->t('Days in Advance'),
      '#default_value' => $this->configuration['days_advance'] ?? 0,
      '#description' => $this->t('Number of days in advance a booking can be made'),
    ];

    $form['advanced_options']['days_visible'] = [
      '#type' => 'number',
      '#title' => $this->t('Days Visible'),
      '#description' => $this->t('Number of days from now a booking can be made'),
    ];

    $form['exclusions'] = [
      '#type' => 'details',
      '#title' => $this->t('Exclusions'),
      '#open' => FALSE,
      '#weight' => 44,
    ];

    $form['exclusions']['exclusion_dates'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Exclusion Dates'),
      '#description' => $this->t('Enter dates in format YYYY-MM-DD or intervals in format YYYY-MM-DD|YYYY-MM-DD, one per line.<br>Ex.<br>@example_date_single<br>@example_date_multi', [
        '@example_date_single' => $this->getExampleDates()['single'],
        '@example_date_multi' => $this->getExampleDates()['multi'],
      ]),
      '#default_value' => $this->configuration['exclusion_dates'] ?? '',
      '#rows' => 3,
      '#element_validate' => [[get_class($this), 'validateExclusionDates']],
    ];

    $form['exclusions']['excluded_weekdays'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Excluded Weekdays'),
      '#options' => [
        'Mon' => $this->t('Mon'),
        'Tue' => $this->t('Tue'),
        'Wed' => $this->t('Wed'),
        'Thu' => $this->t('Thu'),
        'Fri' => $this->t('Fri'),
        'Sat' => $this->t('Sat'),
        'Sun' => $this->t('Sun'),
      ],
      '#default_value' => $this->configuration['excluded_weekdays'] ?? [],
    ];

    // Add new option for time period exclusions.
    $form['exclusions']['excluded_time_periods'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Excluded Time Periods'),
      '#description' => $this->t('Enter time periods in 24h format HH:MM|HH:MM, one per line. You can also add exceptions for specific days (weekdays in English) or dates (YYYY-MM-DD).<br>Ex.<br>12:00|13:00<br>15:30|16:30(Friday)<br>14:00|15:00(2024-10-04)'),
      '#default_value' => $this->configuration['excluded_time_periods'] ?? '',
      '#element_validate' => [[get_class($this), 'validateExcludedTimePeriods']],
    ];

    $form['labels'] = [
      '#type' => 'details',
      '#title' => $this->t('Labels/Messages'),
      '#open' => FALSE,
      '#weight' => 45,
    ];

    $form['labels']['date_label'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Date Label'),
      '#description' => $this->t('Custom label for the date selection. Leave empty to use default.'),
      '#default_value' => $this->configuration['date_label'] ?? '',
    ];

    $form['labels']['slot_label'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Slot Label'),
      '#description' => $this->t('Custom label for the time slot selection. Leave empty to use default.'),
      '#default_value' => $this->configuration['slot_label'] ?? '',
    ];

    $form['labels']['seats_label'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Seats Label'),
      '#description' => $this->t('Custom label for the seats selection. Leave empty to use default.'),
      '#default_value' => $this->configuration['seats_label'] ?? '',
    ];

    $form['labels']['no_slots'] = [
      '#type' => 'textarea',
      '#title' => $this->t('No slots available message'),
      '#description' => $this->t('Insert the text that will be displayed when there are no slots available.'),
      '#default_value' => $this->configuration['no_slots'] ?? '',
      '#rows' => 3,
    ];

    // Add PayPal integration settings.
    $form['payment_settings'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Payment Settings'),
      '#weight' => 46,
    ];

    $form['payment_settings']['paypal_enabled'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable PayPal integration'),
      '#default_value' => $this->configuration['paypal_enabled'] ?? FALSE,
    ];

    $form['payment_settings']['default_price'] = [
      '#type' => 'number',
      '#title' => $this->t('Default Price per Seat'),
      '#default_value' => $this->configuration['default_price'] ?? 0,
      '#min' => 0,
      '#step' => 0.01,
      '#states' => [
        'visible' => [
          ':input[name="properties[paypal_enabled]"]' => ['checked' => TRUE],
        ],
      ],
    ];

    // Adjust weights of generic details.
    if (isset($form['element_description'])) {
      $form['element_description']['#weight'] = 47;
    }
    if (isset($form['validation'])) {
      $form['validation']['#weight'] = 48;
    }

    return $form;
  }

  /**
   * Generate example dates for exclusion dates.
   *
   * @return array
   *   An associative array with example single and multi dates.
   */
  protected static function getExampleDates() {
    $example_date_single = date('Y-m-d');
    $example_date_multi = date('Y-m-d', strtotime('first day of next month')) . '|' . date('Y-m-d', strtotime('last day of next month'));
    return [
      'single' => $example_date_single,
      'multi' => $example_date_multi,
    ];
  }

  /**
   * Validate time intervals.
   */
  public static function validateTimeIntervals(array &$element, FormStateInterface $form_state) {
    $time_intervals = $form_state->getValue($element['#parents']);

    if (is_array($time_intervals)) {
      $time_intervals = reset($time_intervals);
    }

    $time_interval_pattern = '/^(\d{1,2}:\d{2})\|(\d{1,2}:\d{2})(\((Monday|Tuesday|Wednesday|Thursday|Friday|Saturday|Sunday)\))?$/';
    $lines = preg_split('/\r\n|\r|\n/', $time_intervals);

    foreach ($lines as $line) {
      $line = trim($line);
      if (!empty($line) && !preg_match($time_interval_pattern, $line)) {
        // Check if the line matches the incomplete interval pattern.
        $incomplete_interval_pattern = '/^(\d{1,2}:\d{2})\|?$/';
        if (preg_match($incomplete_interval_pattern, $line)) {
          $error_message = t('Incomplete Time Interval. Both start and end times are required in the format HH:MM|HH:MM. Example: 08:00|12:00');
        }
        else {
          $error_message = t('Invalid Time Intervals. Enter time intervals in 24h format HH:MM|HH:MM, one interval per line. You can also add exceptions for specific days or weekdays (weekdays in English). Ex: 10:00|12:00(Friday) or 10:00|12:00(2024-10-04)');
        }
        $form_state->setError($element, $error_message);
        break;
      }
    }
  }

  /**
   * Validate exclusion dates.
   */
  public static function validateExclusionDates(array &$element, FormStateInterface $form_state) {
    $exclusion_dates = $form_state->getValue(['properties', 'exclusion_dates']);

    // Ensure $exclusion_dates is a string.
    if (is_array($exclusion_dates)) {
      $exclusion_dates = reset($exclusion_dates);
    }

    $date_pattern = '/^(\d{4}-\d{2}-\d{2})(\|\d{4}-\d{2}-\d{2})?$/';
    $lines = preg_split('/\r\n|\r|\n/', $exclusion_dates);
    foreach ($lines as $line) {
      if (!empty($line) && !preg_match($date_pattern, $line)) {
        $example_dates = self::getExampleDates();
        $error_message = t('Invalid Exclusion Dates. Enter dates in format YYYY-MM-DD or intervals in format YYYY-MM-DD|YYYY-MM-DD, one per line.<br>Ex.<br>@example_date_single<br>@example_date_multi', [
          '@example_date_single' => $example_dates['single'],
          '@example_date_multi' => $example_dates['multi'],
        ]);
        $form_state->setError($element, $error_message);
        break;
      }
    }
  }

  /**
   * Validate excluded time periods.
   */
  public static function validateExcludedTimePeriods(array &$element, FormStateInterface $form_state) {
    $excluded_time_periods = $form_state->getValue($element['#parents']);

    $time_period_pattern = '/^(\d{1,2}:\d{2})\|(\d{1,2}:\d{2})(\((Monday|Tuesday|Wednesday|Thursday|Friday|Saturday|Sunday|\d{4}-\d{2}-\d{2})\))?$/';
    $lines = preg_split('/\r\n|\r|\n/', $excluded_time_periods);

    foreach ($lines as $line) {
      $line = trim($line);
      if (!empty($line) && !preg_match($time_period_pattern, $line)) {
        $form_state->setError($element, t('Invalid Excluded Time Periods. Enter time periods in 24h format HH:MM|HH:MM, one per line. You can also add exceptions for specific days (weekdays in English) or dates (YYYY-MM-DD).<br>Ex.<br>12:00|13:00<br>15:30|16:30(Friday)<br>14:00|15:00(2024-10-04)'));
        break;
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function prepare(array &$element, WebformSubmissionInterface $webform_submission = NULL) {
    // Ensure the library is attached only once.
    $element['#attached']['library'][] = 'webform_booking/webform_booking';
    $user = \Drupal::currentUser();

    $elementId = $element['#webform_key'];

    if (!isset($element['#attached']['drupalSettings']['webform_booking']['elements'])) {
      $element['#attached']['drupalSettings']['webform_booking']['elements'] = [];
    }

    // Use element ID as a key to ensure unique settings for each element.
    $config = $this->configFactory->get('webform_booking.settings');
    $element['#attached']['drupalSettings']['webform_booking']['elements'][$elementId] = [
      'formId' => $element['#webform'],
      'elementId' => $elementId,
      'startDate' => $element['#start_date'] ?? '',
      'endDate' => $element['#end_date'] ?? '',
      'noSlots' => $element['#no_slots'] ?? '',
      'paypalEnabled' => $element['#paypal_enabled'] ?? FALSE,
      'defaultPrice' => is_numeric($element['#default_price']) ? (float) $element['#default_price'] : 0,
      'currency' => $config->get('currency') ?? 'GBP',
      'maxSeatsPerBooking' => $element['#max_seats_per_booking'] ?? 1,
      'dateLabel' => !empty($element['#date_label']) ? $element['#date_label'] : '',
      'slotLabel' => !empty($element['#slot_label']) ? $element['#slot_label'] : '',
      'seatsLabel' => !empty($element['#seats_label']) ? $element['#seats_label'] : '',
    ];

    $element['#attached']['drupalSettings']['webform_booking']['paypalClientId'] = $config->get('paypal_client_id');
    $element['#attached']['drupalSettings']['webform_booking']['paypalEnvironment'] = $config->get('paypal_environment');
    $element['#attached']['drupalSettings']['webform_booking']['currency'] = $config->get('currency') ?? 'GBP';
    $element['#attached']['drupalSettings']['webform_booking']['paypalDefaultCountry'] = $config->get('paypal_default_country') ?? 'GB';

    // Set up the HTML structure for the calendar and slots.
    $element['#description'] = '<div id="appointment-wrapper-' . $elementId . '"><div id="calendar-container-' . $elementId . '"></div><div id="slots-container-' . $elementId . '"></div>';
    $element['#description'] .= '<div id="seats-dropdown-container-' . $elementId . '"></div>';
    $element['#description'] .= '<div id="price-display-' . $elementId . '"></div>';
    $element['#description'] .= '</div>';

    $element['slot'] = [
      '#type' => $user->hasPermission('view webform booking input') ? 'textfield' : 'hidden',
      '#attributes' => ['id' => 'selected-slot-' . $elementId],
    ];

    $element['seats'] = [
      '#type' => 'hidden',
      '#attributes' => ['id' => 'seats-' . $elementId],
    ];

    if (!$user->hasPermission('edit any webform submission')) {
      $element['#attributes']['class'][] = 'hide';
    }

    // Add custom validation for empty slots.
    $element['#element_validate'][] = [get_class($this), 'validateEmptySlot'];

    return $element;
  }

  /**
   * Add a highlight class to the description if a required field is empty.
   *
   * @param array $element
   *   Element to validate.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   */
  public static function validateEmptySlot(array &$element, FormStateInterface $form_state) {
    $value = $form_state->getValue($element['#parents']);
    if (!empty($element['#required']) && (empty($value) || empty($value['slot']))) {
      $elementId = $element['#webform_key'];
      $element['#description'] = '<div id="appointment-wrapper-' . $elementId . '"><div id="calendar-container-' . $elementId . '"></div><div id="slots-container-' . $elementId . '" class="highlight"></div></div>';

      $form_state->setErrorByName($element['#name'], t('Please select a slot'));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function preSave(array &$element, WebformSubmissionInterface $webform_submission, $update = FALSE) {
    $key = $element['#webform_key'];
    $value = $webform_submission->getElementData($key);

    if (is_array($value) && isset($value['slot'])) {
      // Convert the array to a string.
      $stringValue = $value['slot'];
      $seats = $value['seats'] ?? '1';
      $stringValue .= '|' . $seats;
      $webform_submission->setElementData($key, $stringValue);
    }
    elseif (is_string($value)) {
      // If it's already a string, ensure it's in the correct format.
      $parts = explode('|', $value);
      if (count($parts) < 2) {
        // Add default quantity if missing.
        $value .= '|1';
      }
      $webform_submission->setElementData($key, $value);
    }
  }

}
